#!/usr/bin/perl -w

use strict;
use Time::Piece;
use Time::Seconds;
use Number::Format qw(:subs);

###############################
### configuration variables ###
###############################
my $error_log = "/var/log/httpd-error.log";         # apache error log
my $window_time = 15;
my $window_thresh = 31;
my $min_total = 100;                                # these three values mean "31 errors in 15 seconds, and at least 100 total errors"

open(ELOG, "<", $error_log) or die("Can't open error log $error_log: $!");

my $events = {};            # hash of arrays of timestamps, keyed by IP

while(<ELOG>) {
    print STDERR "\r$." if $. % 1000 == 0;
    
    if( m!^\[([\w\d: ]+)\] +\[error\] +\[client +(\d+\.\d+\.\d+\.\d+)\] +(.*)$! ) {
        my ($timestamp, $ipaddr, $reason) = ($1, $2, $3);
        
#        print STDERR "debug: \$timestamp = \'$timestamp\' \$ipaddr = \'$ipaddr\'\n";
        my $ts = Time::Piece->strptime($timestamp, "%a %b %e %H:%M:%S %Y");  # example: Sat Sep 27 13:58:58 2003
        push @{ $events->{$ipaddr} }, $ts->epoch;
    }
}

close(ELOG);

my $infrac = {};

foreach my $ip (keys %$events) {
    my $num = scalar( @{ $events->{$ip} } );
    my $n_infrac = 0;

    next unless( $num >= $window_thresh && $num >= $min_total );
    
    for(my $i = 0; $i + $window_thresh <= $num; $i++) {
        
        my ($first, $last) = ($events->{$ip}->[$i], $events->{$ip}->[$i + $window_thresh - 1]);
        
        if( $last - $first > $window_time ) {
            $n_infrac++;
        }
    }
    
    $infrac->{$ip} = $n_infrac unless ( $n_infrac == 0 );
}

foreach my $ip (sort { $infrac->{$a} <=> $infrac->{$b} } keys %$infrac) {
    printf "\t# %d infractions (%d total errors)\n\tRewriteCond %%\{REMOTE_ADDR\} ^%s\$\n", $infrac->{$ip}, scalar( @{ $events->{$ip} } ), quotemeta($ip);
}
