#!/usr/bin/perl -w

use Parse::AccessLogEntry;
use Time::Piece;
use Date::Parse;
use Data::Dumper;

#  $Hashref->{host}    client ip of the request
#  $Hashref->{user}    user logged in ("-" for none)
#  $Hashref->{date}    date of the request
#  $Hashref->{time}    server time of the request
#  $Hashref->{diffgmt} server offset from GMT
#  $Hashref->{rtype}   type of request (GET, POST, etc)
#  $Hashref->{file}    file requested
#  $Hashref->{proto}   protocol used (HTTP/1.1, etc)
#  $Hashref->{code}    code returned by apache (200, 304, etc)
#  $Hashref->{bytes}   number of bytes returned to the client
#  $Hashref->{refer}   referrer
#  $Hashref->{agent}   user-agent

my $p = Parse::AccessLogEntry::new();

my @exclude_hosts = (
                        "203.194.198.77", 
                        "203.194.198.78", 
                        "127.0.0.1", 
                    );

my $ts = {};
my $ev = {};
my $ua = {};
my $ipaddr = {};

while( <> ) {
    print STDERR "$.\r" if $. % 1000 == 0;
	my $l = $p->parse( $_ );

#	next if grep {$l->{host} eq $_} @exclude_hosts;

    next unless($l->{file} =~ m!/announce\.php\?.*info_hash=([^&]+)&.*peer_id=([^&]+)&!i);

    my $client_id = $1 . $2;

    my $event = "";
    if ($l->{file} =~ m!\&event=([\w]+)!) {                 # one of 'started', 'completed', 'stopped'
        $event = $1;
    }

    $client_id =~ s/%([A-Fa-f\d]{2})/chr hex $1/eg;
    
    $client_id = join("", map( sprintf( "%02x", ord $_ ), split("", $client_id)));

    my $tstamp = Time::Piece->strptime($l->{date} . " " . $l->{time}, "%d/%b/%Y %H:%M:%S");   
#    my $tstamp = str2time($l->{date} . " " . $l->{time} . " " . $l->{diffgmt});

    push @{ $ts->{$client_id} }, $tstamp->epoch;
    push @{ $ev->{$client_id} }, $event;
    $ua->{$client_id} = $l->{agent};
    $ipaddr->{$client_id} = $l->{host};

}

#print Dumper $ts;
#print Dumper $ua;

#exit 0;

foreach my $clid ( sort { $ua->{$a} cmp $ua->{$b} } keys %$ua ) {
    my $tslist = $ts->{$clid};
    my $evlist = $ev->{$clid};
   
    if( scalar( @$tslist ) >= 2) {
    
        my @deltas = ();
        for( my $i = 1; $i < scalar( @$tslist ); $i++) {
            push @deltas, ($tslist->[$i] - $tslist->[$i - 1]);
        }
        
        printf "%15.15s %-60.60s %s\n", $ipaddr->{$clid}, $ua->{$clid}, mkeventlist($tslist, $evlist);
    }
}        
        
        
sub avg {
    # passed an array ref this returns its average
    my $aref = shift;
    return undef if(!defined($aref) || scalar(@$aref) < 1);
            
    my $tot = 0;
                    
    map { $tot += $_; } @$aref;
                            
    return scalar(@$aref) ? ($tot / scalar(@$aref)) : undef;
}

sub mkeventlist {
    # passed an array ref of epoch timestamps, and an arrayref of event labels (most as "")
    # this returns a formatted string of events
    my ($tslist, $evlist) = @_;
    my @deltas = ();
    my $output = "";
    
    return "" if( scalar( @$tslist ) < 2 || scalar( @$tslist ) != scalar( @$evlist ) );
    
    for (my $i = 1; $i < scalar( @$tslist ); $i++) {
        if( $evlist->[$i - 1] ) {
            $output .= $evlist->[$i - 1] . " ";
        }
        my $thisdelta = ($tslist->[$i] - $tslist->[$i - 1]);
        $output .= "$thisdelta ";
        if( $evlist->[$i] ) {
            $output .= $evlist->[$i] . " ";
        }
        push @deltas, $thisdelta unless ( $evlist->[$i] eq "stopped" || $evlist->[$i] eq "completed" );
    }
    
    if ( @deltas > 1 ) {
        return sprintf "avg: %-5d (%s)", avg(\@deltas), $output;
    }
    return "avg: - ($output)";
}
